/*
 * io.c
 *
 *  Created on: 24.12.2014
 *      Author: AndreR
 */

#include "io.h"
#include "util/intercom.h"
#include "intercom_constants.h"
#include "hal/motor.h"
#include "supply.h"
#include "hal/kicker.h"
#include "kick.h"
#include "log_msgs.h"

IOData io;

static void ctrlCallback(const IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)pUserData;
	(void)dataLength;

	if(io.ignoreMainCmds)
		return;

	switch(pHeader->command)
	{
		case IC_KD_FIRE:
		{
			KdFire* pFire = (KdFire*)pData;

			switch(pFire->mode)
			{
				case KICKER_MODE_FORCE: KickFireSpeed(pFire->speed, pFire->device); break;
				case KICKER_MODE_ARM: KickArmSpeed(pFire->speed, pFire->device); break;
				case KICKER_MODE_ARM_TIME: KickArm(pFire->speed, pFire->device); break;
				default: KickDisarm(); break;
			}
		}
		break;
		case IC_KD_DRIBBLER:
		{
			DribblerInput* pDrib = (DribblerInput*)pData;

			if((pDrib->speed > DRIVE_MOVE_THRESHOLD || pDrib->speed < -DRIVE_MOVE_THRESHOLD) && supply.batEmpty == 0)
			{
				MotorSpeedSetVelocity(pDrib->speed*2.2f);
			}
			else
			{
				MotorSpeedSetVelocity(0);
			}
		}
		break;
		case IC_KD_AUTOCHG:
		{
			KdAutoCharge* pChg = (KdAutoCharge*)pData;

			if(pChg->enable && supply.batEmpty == 0)
			{
				KickerEnableAutoRecharge(1);
				KickerEnableCharge(1);
			}
			else
			{
				KickDischarge();
			}
		}
		break;
	}
}

void IoInit()
{
	IntercomClient iClient;
	iClient.pUserData = 0;
	iClient.rxCallback = &ctrlCallback;
	iClient.section = INTERCOM_SECTION_KD;

	IntercomAddClient(iClient);
}

void IoTask(void* params)
{
	(void)params;

	uint32_t lastPowerGoodTime = chVTGetSystemTime();
	IntercomHeader iHeader;
	KdFeedback kdFeedback;

	iHeader.address = INTERCOM_ADDRESS_MAIN | INTERCOM_ADDRESS_MEDIA;
	iHeader.section = INTERCOM_SECTION_KD;
	iHeader.command = IC_KD_FEEDBACK;

	chRegSetThreadName("I/O");

	while(1)
	{
		chThdSleepMilliseconds(5);

		if(supply.bat > 10.5f)
			lastPowerGoodTime = chVTGetSystemTime();

		if(chVTTimeElapsedSinceX(lastPowerGoodTime) > S2ST(10) && supply.batEmpty == 0)
		{
			supply.batEmpty = 1;

			KickDischarge();
		}

		kdFeedback.kicker.cap = kicker.vCap;
		kdFeedback.kicker.chg = kicker.iChg;
		kdFeedback.kicker.state = kicker.state | (kicker.autoCharge << 8) | (kicker.autoDischarge << 12);
		kdFeedback.kicker.kickCounter = kicker.kickCounter;
		if(kicker.vCap > kick.config.maxVoltage*kick.config.kickVoltageLimit)
			kdFeedback.kicker.ready = 1;
		else
			kdFeedback.kicker.ready = 0;

		kdFeedback.dribbler.velocity = motor.curVelocity;
		kdFeedback.dribbler.temp = motor.temp;
		kdFeedback.dribbler.overheated = motor.overheated;

		if(supply.irResult[0] > supply.irResult[1])
		{
			kdFeedback.barrier.level[0] = supply.irResult[0];
			kdFeedback.barrier.level[1] = supply.irResult[1];
		}
		else
		{
			kdFeedback.barrier.level[0] = supply.irResult[1];
			kdFeedback.barrier.level[1] = supply.irResult[0];
		}
		kdFeedback.barrier.interrupted = kick.interrupted;
		kdFeedback.barrier.damaged = kick.barrierDamaged;

		kdFeedback.power.bat = supply.bat;
		kdFeedback.power.cur = supply.shunt;
		kdFeedback.power.batEmpty = supply.batEmpty;

//		kdFeedback.config.maxCapLevel = kick.config.maxVoltage;
//		kdFeedback.config.cooldown = kick.config.cooldown;
//		kdFeedback.config.irThreshold = kick.config.irThreshold;
//		kdFeedback.config.irFilterLimit = kick.config.irFilterThreshold;

		IntercomSend(&iHeader, sizeof(KdFeedback), &kdFeedback);
	}
}
