/*
 * touch.c
 *
 *  Created on: 05.07.2014
 *      Author: AndreR
 */

#include "touch.h"
#include "../constants.h"
#include "util/bits.h"
#include "util/init_hal.h"
#include "util/console.h"
#include <math.h>
#include <string.h>

#include "gfx.h"
#include "src/ginput/ginput_driver_mouse.h"

// Resolution and Accuracy Settings
#define GMOUSE_MAX11802_PEN_CALIBRATE_ERROR		8
#define GMOUSE_MAX11802_PEN_CLICK_ERROR			16
#define GMOUSE_MAX11802_PEN_MOVE_ERROR			4
#define GMOUSE_MAX11802_FINGER_CALIBRATE_ERROR	14
#define GMOUSE_MAX11802_FINGER_CLICK_ERROR		18
#define GMOUSE_MAX11802_FINGER_MOVE_ERROR		14

#define Z_MIN		0
#define Z_MAX		1

Touch touch;

// touch event active
void EXTI2_IRQHandler()
{
	CH_IRQ_PROLOGUE();

	EXTI->PR = EXTI_PR_PR2;

	chSysLockFromISR();
//	chBSemSignalI(&touch.touchedSem);
	chSysUnlockFromISR();

	CH_IRQ_EPILOGUE();
}

void DMA1_Stream2_IRQHandler()
{
	CH_IRQ_PROLOGUE();

	SPISyncDmaRxInterrupt(&touch.spi3);

	CH_IRQ_EPILOGUE();
}

void TouchInit()
{
	RCC->APB1ENR |= RCC_APB1ENR_SPI3EN;

	SPISyncData data3;
	data3.dmaChRx = 2;
	data3.dmaChTx = 7;
	data3.dmaPrio = 1;
	data3.pDma = DMA1;
	data3.pRegister = SPI3;
	SPISyncInit(&touch.spi3, &data3);

	touch.slave.cpha = 0;
	touch.slave.cpol = 0;
	touch.slave.csPin = GPIO_PIN_6;
	touch.slave.pCSPort = GPIOD;
	touch.slave.timeoutTicks = MS2ST(20);
	touch.slave.prescaler = SPI_CR1_BRDIV8;
	SPISyncSlaveInit(&touch.spi3, &touch.slave);

	NVICEnableIRQ(DMA1_Stream2_IRQn, IRQL_SPI3);

	GPIOInitData gpioInit;
	gpioInit.mode = GPIO_MODE_AF;
	gpioInit.alternate = 6;	// SPI3
	gpioInit.otype = GPIO_OTYPE_PUSH_PULL;
	gpioInit.ospeed = GPIO_OSPEED_25MHZ;
	gpioInit.pupd = GPIO_PUPD_UP;
	GPIOInit(GPIOC, GPIO_PIN_10 | GPIO_PIN_11 | GPIO_PIN_12, &gpioInit);	// SCK, MISO, MOSI

	// TP-IRQ: E2
	gpioInit.mode = GPIO_MODE_EXTI;
	gpioInit.extiTrigger = GPIO_EXTI_TRIG_RISING_FALLING;
	GPIOInit(GPIOE, GPIO_PIN_2, &gpioInit);

//	NVICEnableIRQ(EXTI2_IRQn, IRQL_EXTI2);

	touch.buf[0]= 0x01 << 1;
	touch.buf[1]= 0xC0;	//

	touch.buf[2] = 0x03 << 1;
	touch.buf[3] = 0x55; // 16 samples, median filter

	touch.buf[4] = 0x04 << 1;
	touch.buf[5] = 0x00; // ADC sample time 128us on all channels

	touch.buf[6] = 0x05 << 1;
	touch.buf[7] = 0x77; // Panel setup time 1ms

	touch.buf[8]= 0x06 << 1;
	touch.buf[9]= 0x55; // Delayed conversion 1ms

	touch.buf[10] = 0x07 << 1;
	touch.buf[11] = 0x00; // set PUR to 1ms

	touch.buf[12] = 0x0B << 1; // write Mode Config Register
	touch.buf[13] = 0x0E; // avg filter on x,y,z1,z2

	SPISyncTransfer(&touch.slave, touch.buf, 0, 14);

	chThdSleepMilliseconds(50);

	SPISyncTransfer(&touch.slave, touch.buf, 0, 14);
}

#if GFX_USE_GINPUT && GINPUT_NEED_MOUSE

static bool_t readXYZ(GMouse* m, GMouseReading* pdr)
{
	(void)m;

	uint8_t readyCount;
	uint8_t notReadyCount;

	// Assume not touched.
	pdr->buttons = 0;
	pdr->z = 0;

	// Start the conversion
	touch.buf[0] = 0x70 << 1;
	SPISyncTransfer(&touch.slave, touch.buf, 0, 1);

    /**
	 * put a delay in here, since conversion will take a finite time - longer if we were reading Z as well
	 * Potentially at least 1msec for 3 axes with 8us conversion time per sample, 8 samples per average
	 * Note Maxim AN5435-software to do calculation (www.maximintegrated.com/design/tools/appnotes/5435/AN5435-software.zip)
	 *
	 * We'll just use a fixed delay to avoid too much polling/bus activity
	 */
	chThdSleepMilliseconds(1);

	/* Wait for data ready
	* Note: MAX11802 signals the readiness of the results using the lowest 4 bits of the result. However, these are the
	* last bits to be read out of the device. It is possible for the hardware value to change in the middle of the read,
	* causing the analog value to still be invalid even though the tags indicate a valid result.
	*
	* We work around this by reading the registers once more after the tags indicate they are ready.
	* There's also a separate counter to guard against never getting valid readings.
	*
	* Read the two readings required in a single burst, swapping x and y order if necessary
	*
	* Reading Z is possible but complicated requiring two z readings, multiplication and division, various constant ratio's,
	* 	and interpolation in relation to the X and Y readings. It is not a simple pressure reading.
	* 	In other words, don't bother trying.
	*/

	readyCount = notReadyCount = 0;
	do
	{
		// Get a set of readings
		touch.buf[0] = (0x52 << 1) | 0x01;
		SPISyncTransfer(&touch.slave, touch.buf, touch.buf, 5);

		uint16_t x = ((uint16_t)touch.buf[1] << 8) | touch.buf[2];
		uint16_t y = ((uint16_t)touch.buf[3] << 8) | touch.buf[4];

//		ConsolePrint("%hu, %hu\r\n", x, y);

		#if defined(GINPUT_MOUSE_YX_INVERTED) && GINPUT_MOUSE_YX_INVERTED
			pdr->y = x;
			pdr->x = y;
		#else
			pdr->x = x;
			pdr->y = y;
		#endif

		// Test the result validity
		if(((pdr->x | pdr->y) & 0x03) == 0x03)
		{
			// Has it been too long? If so give up
			if(++notReadyCount >= 10)
				return FALSE;

			// Give up the time slice to someone else and then try again
			chThdSleepMilliseconds(1);
			chThdYield();
			continue;
		}

		// We have a result but we need two valid results to believe it
		readyCount++;

	} while(readyCount < 2);


	/**
	 *	format of each value returned by MAX11802:
	 *		Bits 15..4 - analog value
	 *		Bits 3..2 - tag value - measurement type (X, Y, Z1, Z2)
	 *		Bits 1..0 - tag value - event type (00 = valid touch, 10 - no touch, 11 - measurement in progress)
	 */

    // Was there a valid touch?
    if(((pdr->x | pdr->y) & 0x03) != 0x0)
    {
    	pdr->z = Z_MIN;

        if(chVTTimeElapsedSinceX(touch.lastTouchTime) > MS2ST(10000))
        {
        	touch.lastTouchTime = chVTGetSystemTime();
        	gdispSetBacklight(30);
        }

    	return TRUE;
    }

    if(chVTTimeElapsedSinceX(touch.lastTouchTime) > MS2ST(10))
		gdispSetBacklight(100);

    touch.lastTouchTime = chVTGetSystemTime();

    // Strip the tags (we need to take care because coord_t is signed - and sign bit gets extended on shift!)
    pdr->x = (uint16_t)(pdr->x) >> 4;
    pdr->y = (uint16_t)(pdr->y) >> 4;
   	pdr->z = Z_MAX;

//   	ConsolePrint("%hd, %hd\r\n", pdr->x, pdr->y);

    return TRUE;
}

static bool_t mouseInit(GMouse* m, unsigned driverinstance)
{
	(void)m;
	(void)driverinstance;

	return TRUE;
}

static void calSave(GMouse *m, const void *buf, size_t sz)
{
	(void)m;
	(void)sz;

	float* pCalib = (float*)buf;

	for(uint8_t i = 0; i < 6; i++)
		ConsolePrint("%.8f\r\n", pCalib[i]);
}

static const float calib[] = {
	0.08277013f, 0.0f, -12.71388721f,
	0.0f, -0.06284435f, 246.20465088f
};

static bool_t calLoad(GMouse *m, void *buf, size_t sz)
{
	(void)m;

	if(sz != sizeof(calib))
		return FALSE;

	memcpy(buf, calib, sz);

//	ConsolePrint("Loaded mouse calib\r\n");

	return TRUE;
}

const GMouseVMT const GMOUSE_DRIVER_VMT[1] = {{
	{
		GDRIVER_TYPE_TOUCH,
		GMOUSE_VFLG_TOUCH | GMOUSE_VFLG_ONLY_DOWN/* | GMOUSE_VFLG_POORUPDOWN*/ | GMOUSE_VFLG_CALIBRATE | GMOUSE_VFLG_CAL_TEST | GMOUSE_VFLG_CAL_EXTREMES,
		sizeof(GMouse),
		_gmouseInitDriver,
		_gmousePostInitDriver,
		_gmouseDeInitDriver
	},
	Z_MAX,			// z_max
	Z_MIN,			// z_min
	Z_MAX,  		// z_touchon
	Z_MIN,			// z_touchoff
	{				// pen_jitter
		GMOUSE_MAX11802_PEN_CALIBRATE_ERROR,		// calibrate
		GMOUSE_MAX11802_PEN_CLICK_ERROR,			// click
		GMOUSE_MAX11802_PEN_MOVE_ERROR				// move
	},
	{				// finger_jitter
		GMOUSE_MAX11802_FINGER_CALIBRATE_ERROR,		// calibrate
		GMOUSE_MAX11802_FINGER_CLICK_ERROR,			// click
		GMOUSE_MAX11802_FINGER_MOVE_ERROR			// move
	},
	mouseInit, 		// init
	0,				// deinit
	readXYZ,		// get
	calSave,		// calsave
	calLoad			// calload
}};

#endif
