/*
 * log_msgs.c
 *
 *  Created on: 29.02.2016
 *      Author: AndreR
 */

#include "log_msgs.h"
#include "struct_ids.h"

const LogMessageDesc logMessageDescriptions[9] = {
	{ SID_GYRO_RAW, "gyro_raw", 3, (ElementDesc[]) {
		{ INT16, "X", "raw", "X axis" },
		{ INT16, "Y", "raw", "Y axis" },
		{ INT16, "Z", "raw", "Z axis" },
	}, },
	{ SID_ACC_RAW, "acc_raw", 3, (ElementDesc[]) {
		{ INT16, "X", "raw", "X axis" },
		{ INT16, "Y", "raw", "Y axis" },
		{ INT16, "Z", "raw", "Z axis" },
	}, },
	{ SID_SENSORS, "sensors", 42, (ElementDesc[]) {
		{ UINT32,	"vel_t", "us",	"Motor velocity timestamp" },
		{ FLOAT,	"vel_m1", "rad/s","Motor 1 angular velocity" },
		{ FLOAT,	"vel_m2", "rad/s","Motor 2 angular velocity" },
		{ FLOAT,	"vel_m3", "rad/s","Motor 3 angular velocity" },
		{ FLOAT,	"vel_m4", "rad/s","Motor 4 angular velocity" },

		{ UINT32,	"gyr_updated", "bool", "Gyroscope data updated" },
		{ UINT32,	"gyr_t", "us", "Timestamp" },
		{ FLOAT,	"gyr_x", "rad/s", "Angular velocity around X axis" },
		{ FLOAT,	"gyr_y", "rad/s", "Angular velocity around Y axis" },
		{ FLOAT,	"gyr_z", "rad/s", "Angular velocity around Z axis" },

		{ UINT32,	"acc_updated", "bool", "Accelerometer data updated" },
		{ UINT32,	"acc_t", "us", "Timestamp" },
		{ FLOAT,	"acc_x", "m/s^2", "Acceleration along X axis" },
		{ FLOAT,	"acc_y", "m/s^2", "Acceleration along Y axis" },
		{ FLOAT,	"acc_z", "m/s^2", "Acceleration along Z axis" },

		{ UINT32,	"pos_updated", "bool", "Position data updated" },
		{ UINT32,	"pos_t", "us", "Timestamp" },
		{ UINT32,	"pos_delay", "us", "Vision Delay" },
		{ FLOAT,	"pos_x", "m", "X Position" },
		{ FLOAT,	"pos_y", "m", "Y Position" },
		{ FLOAT,	"pos_z", "m", "Z Position" },
		{ UINT32,	"cam_id", "-", "Camera ID" },
		{ UINT32,	"no_vision", "-", "No Vision Flag" },

		{ FLOAT,	"supply_bat", "V", "Battery voltage" },
		{ FLOAT,	"supply_cur", "A", "Current consumption" },
		{ UINT32,	"bat_empty", "bool", "Battery empty" },

		{ UINT32,	"vol_t", "us", "Motor voltage timestamp" },
		{ FLOAT,	"vol_m1", "V", "Motor 1 voltage" },
		{ FLOAT,	"vol_m2", "V", "Motor 2 voltage" },
		{ FLOAT,	"vol_m3", "V", "Motor 3 voltage" },
		{ FLOAT,	"vol_m4", "V", "Motor 4 voltage" },

		{ FLOAT,	"kicker_level", "V", "Kicker Level" },
		{ FLOAT,	"kicker_chg", "A", "Kicker charge current" },
		{ UINT32,	"kicker_counter", "-",	"Kick Counter" },
		{ UINT32,	"kicker_flags", "-",	"Status flags" },

		{ FLOAT,	"dribbler_speed", "RPM", "Dribbler speed" },
		{ FLOAT,	"dribbler_temp", "C", "Dribbler temperature" },
		{ FLOAT,	"dribbler_voltage", "V", "Dribbler voltage" },
		{ UINT32,	"dribbler_overheated", "bool",	"Dribbler overheated" },

		{ FLOAT,	"ir_on", "V", "IR Voltage Sender On" },
		{ FLOAT,	"ir_off", "V", "IR Voltage Sender Off" },
		{ UINT32,	"ir_interrupted", "bool", "Barrier interrupted" },
	}, },
	{ SID_CTRL_STATE, "ctrl_state", 12, (ElementDesc[]) {
		{ FLOAT,	"pos_x", "m", "X Position" },
		{ FLOAT,	"pos_y", "m", "Y Position" },
		{ FLOAT,	"pos_z", "rad", "Z Position" },

		{ FLOAT,	"vel_x", "m/s", "X Velocity" },
		{ FLOAT,	"vel_y", "m/s", "Y Velocity" },
		{ FLOAT,	"vel_z", "rad/s", "Z Velocity" },

		{ FLOAT,	"acc_x", "m/s^2", "X Acceleration" },
		{ FLOAT,	"acc_y", "m/s^2", "Y Acceleration" },
		{ FLOAT,	"acc_z", "rad/s^2", "Z Acceleration" },

		{ UINT32,	"pos_updated", "bool", "Position estimation updated" },
		{ UINT32,	"vel_updated", "bool", "Velocity estimation updated" },
		{ UINT32,	"acc_updated", "bool", "Acceleration estimation updated" },
	}, },
	{ SID_CTRL_OUTPUT, "ctrl_output", 13, (ElementDesc[]) {
		{ FLOAT,	"vel_m1", "rad/s", "Motor 1 velocity" },
		{ FLOAT,	"vel_m2", "rad/s", "Motor 2 velocity" },
		{ FLOAT,	"vel_m3", "rad/s", "Motor 3 velocity" },
		{ FLOAT,	"vel_m4", "rad/s", "Motor 4 velocity" },
		{ FLOAT,	"vol_m1", "V", "Motor 1 voltage" },
		{ FLOAT,	"vol_m2", "V", "Motor 2 voltage" },
		{ FLOAT,	"vol_m3", "V", "Motor 3 voltage" },
		{ FLOAT,	"vol_m4", "V", "Motor 4 voltage" },
		{ FLOAT,	"acc_m1", "rad/s^2", "Motor 1 acceleration" },
		{ FLOAT,	"acc_m2", "rad/s^2", "Motor 2 acceleration" },
		{ FLOAT,	"acc_m3", "rad/s^2", "Motor 3 acceleration" },
		{ FLOAT,	"acc_m4", "rad/s^2", "Motor 4 acceleration" },
		{ UINT32,	"ctrlMode", "-", "Motor controller mode"},
	}, },
	{ SID_CTRL_TIGGA_DEBUG, "ctrl_tigga_debug", 2, (ElementDesc[]) {
		{ UINT32,	"fusion_time", "us", "Sensor fusion processing time" },
		{ UINT32,	"ctrl_time", "us", "Control processing time" },
	}, },
	{ SID_CTRL_REF, "ctrl_ref", 12, (ElementDesc[]) {
		{ FLOAT,	"traj_pos_x", "m", "Global Position from Trajectory X" },
		{ FLOAT,	"traj_pos_y", "m", "Global Position from Trajectory Y"},
		{ FLOAT,	"traj_pos_z", "m", "Global Position from Trajectory Z" },

		{ FLOAT,	"traj_vel_local_x", "m/s", "Local Velocity from Trajectory X" },
		{ FLOAT,	"traj_vel_local_y", "m/s", "Local Velocity from Trajectory Y"},
		{ FLOAT,	"traj_vel_local_z", "m/s", "Local Velocity from Trajectory Z" },

		{ FLOAT,	"traj_acc_local_x", "m/s^2", "Local Acceleration from Trajectory X" },
		{ FLOAT,	"traj_acc_local_y", "m/s^2", "Local Acceleration from Trajectory Y" },
		{ FLOAT,	"traj_acc_local_z", "m/s^2", "Local Acceleration from Trajectory Z" },

		{ FLOAT,	"traj_jerk_local_x", "m/s^3", "Local Jerk from Trajectory X" },
		{ FLOAT,	"traj_jerk_local_y", "m/s^3", "Local Jerk from Trajectory Y" },
		{ FLOAT,	"traj_jerk_local_z", "m/s^3", "Local Jerk from Trajectory Z" },
	}, },
	{ SID_OPTICAL_RAW, "optical_raw", 6, (ElementDesc[]) {
		{ INT16,	"l_delta_x", "-", "Left Delta X" },
		{ INT16,	"l_delta_y", "-", "Left Delta Y" },
		{ UINT8,	"l_squal", "-", "Left Quality" },
		{ INT16,	"r_delta_x", "-", "Right Delta X" },
		{ INT16,	"r_delta_y", "-", "Right Delta Y" },
		{ UINT8,	"r_squal", "-", "Right Quality" },
	}, },
	{ SID_SKILL_OUTPUT, "skill_output", 24, (ElementDesc[]) {
		{ FLOAT,	"pos_x", "m", "X Position" },
		{ FLOAT,	"pos_y", "m", "Y Position" },
		{ FLOAT,	"pos_z", "rad", "Z Position" },

		{ FLOAT,	"vel_x", "m/s", "X Velocity Local" },
		{ FLOAT,	"vel_y", "m/s", "Y Velocity Local" },
		{ FLOAT,	"vel_z", "rad/s", "Z Velocity Local" },

		{ FLOAT,	"vel_w1", "rad/s", "Wheel 1 velocity" },
		{ FLOAT,	"vel_w2", "rad/s", "Wheel 2 velocity" },
		{ FLOAT,	"vel_w3", "rad/s", "Wheel 3 velocity" },
		{ FLOAT,	"vel_w4", "rad/s", "Wheel 4 velocity" },

		{ UINT32,	"mode_xy", "-", "XY control mode" },
		{ UINT32,	"mode_w", "-", "W control mode" },

		{ FLOAT,	"vel_max_xy", "m/s", "Max. velocity XY" },
		{ FLOAT,	"acc_max_xy", "m/s^2", "Max. acceleration XY" },
		{ FLOAT,	"jerk_max_xy", "m/s^3", "Max. jerk XY" },
		{ FLOAT,	"vel_max_w", "rad/s", "Max. velocity W" },
		{ FLOAT,	"acc_max_w", "rad/s^2", "Max. acceleration W" },
		{ FLOAT,	"jerk_max_w", "rad/s^3", "Max. jerk W" },

		{ FLOAT,	"dribbler_speed", "RPM", "Dribbler RPM" },
		{ FLOAT,	"dribbler_voltage", "V", "Dribbler Voltage" },
		{ UINT32,	"dribbler_mode", "-", "Dribbler control mode" },

		{ UINT32,	"kicker_mode", "-", "Kicker control mode" },
		{ UINT32,	"kicker_device", "-", "Kicker device" },
		{ FLOAT,	"kicker_speed", "m/s", "Kick speed" },
	}, },
};
